------------------------------------------------------------------------------
--                                                                          --
--                 GNU ADA RUNTIME LIBRARY (GNARL) COMPONENTS               --
--                                                                          --
--               S Y S T E M . T A S K I N G . A B O R T I O N              --
--                                                                          --
--                                  S p e c                                 --
--                                                                          --
--                             $Revision: 1.14 $                            --
--                                                                          --
--     Copyright (c) 1991,1992,1993,1994,1995 FSU, All Rights Reserved      --
--                                                                          --
-- GNARL is free software; you can redistribute it  and/or modify it  under --
-- terms  of  the  GNU  Library General Public License  as published by the --
-- Free Software  Foundation;  either version 2, or (at  your  option)  any --
-- later  version.  GNARL is distributed  in the hope that  it will be use- --
-- ful, but but WITHOUT ANY WARRANTY;  without even the implied warranty of --
-- MERCHANTABILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Gen- --
-- eral Library Public License  for more details.  You should have received --
-- a  copy of the GNU Library General Public License along with GNARL;  see --
-- file COPYING.LIB.  If not,  write to the  Free Software Foundation,  675 --
-- Mass Ave, Cambridge, MA 02139, USA.                                      --
--                                                                          --
------------------------------------------------------------------------------

with System.Task_Primitives;
--  Used for,  Task_Primitives.Pre_Call_State

package System.Tasking.Abortion is

   procedure Abort_Tasks (Tasks : Task_List);
   --  Abort_Tasks is called to initiate abortion, however, the actual
   --  abortion is done by abortee by means of Abort_Handler

   procedure Change_Base_Priority (T : Task_ID);
   --  Change the base priority of T.
   --  Has to be called with T.Lock write locked.

   procedure Defer_Abortion;
   --  Defer the affects of low-level abortion in the calling task until a
   --  matching Undefer_Abortion call is executed.  Defer_Abortion can be
   --  nested; abortion will be deferred until the calling task has
   --  called Undefer_Abortion for each outstanding call to
   --  Defer_Abortion.  Note that abortion must be deferred before
   --  calling any low-level (GNULLI) services.
--    pragma Inline (Defer_Abortion); --  To allow breakpoints to be set. ???

   procedure Undefer_Abortion;
   --  Undo the effects of one call to Defer_Abortion.  When the calling
   --  task has called Undefer_Abortion for each outstanding call to
   --  Defer_Abortion, any pending low-level abortion will take effect,
   --  and subsequent low-level abortions will have an immediate
   --  asynchronous effect.
--    pragma Inline (Undefer_Abortion); --  To allow breakpoints to be set.

end System.Tasking.Abortion;
